-- Basic Co-op 1.2 by Gregory Smith
-- Copyright 2020
-- GPL 3 (https://www.gnu.org/licenses/gpl-3.0.en.html)

Triggers = {}

function suspend_triggers()
   SuspendedTriggers = Triggers
   Triggers = {}
end

function restore_triggers()
   Triggers = SuspendedTriggers
   SuspendedTriggers = {}
end

function exit_with_message(message)
   Triggers = {}
   Triggers.idle = function()
      for p in Players() do
         p:print(message)
      end
      Triggers = {}
   end
end

function Triggers.init()
   if Game.version <= "20150619" then
      exit_with_message("Basic co-op requires Aleph One 1.3")
      return
   end

   if Game.type ~= "cooperative play" then
      exit_with_message("Basic co-op only works with cooperative games!")
      return
   end

   Game.proper_item_accounting = true
   Game.dead_players_drop_items = false -- bury me with my loot
end

function Triggers.idle()
   for p in Players() do
      if p.action_flags.action_trigger then
         local target = p:find_action_key_target()
         if is_side(target)
            and target.control_panel
            and target.control_panel.type.class == "pattern buffer"
         then
            for pp in Players() do
               pp._save = {}
               pp._save.x = p.x
               pp._save.y = p.y
               pp._save.z = p.z
               pp._save.yaw = p.yaw
               pp._save.polygon = p.polygon
            end
         end
      end
   end
end

function is_weapon(item)
   -- might not necessarily work with all scenarios; Aleph One needs to support
   -- getting the kind of an item
   return
      item == "knife" or
      item == "pistol" or
      item == "fusion pistol" or
      item == "assault rifle" or
      item == "missile launcher" or
      item == "alien weapon" or
      item == "flamethrower" or
      item == "shotgun" or
      item == "smg"
end

function is_ammo(item)
   -- might not necessarily work with all scenarios; Aleph One needs to support
   -- getting the kind of an item
   return
      item == "pistol ammo" or
      item == "fusion pistol ammo" or
      item == "assault rifle ammo" or
      item == "assault rifle grenades" or
      item == "missile launcher ammo" or
      item == "alien weapon ammo" or
      item == "flamethrower ammo" or
      item == "shotgun ammo" or
      item == "smg ammo"
end

function Triggers.got_item(item, player)
   if is_weapon(item) or is_ammo(item) then
      suspend_triggers()
      for p in Players() do
         if p ~= player and not p.dead then
            p.items[item] = p.items[item] + 1
         end
      end
      restore_triggers()
   end
end

function Triggers.player_killed(player)
   player._stuff = {}
   for item in ItemTypes() do
      player._stuff[item] = player.items[item]
      if player.items[item] > 0 and
         not is_ammo(item) and
         not is_weapon(item)
      then
         -- DPDI still drops items with kind == item, so burn it here
         player.items[item] = 0
      end
   end
end

function is_flooded(polygon)
   return polygon.media and
      (polygon.media.type == "lava" or polygon.media.type == "goo") and
      polygon.media.height > polygon.floor.height
end

function Triggers.player_revived(player)
   suspend_triggers()
   for item in ItemTypes() do
      player.items[item] = player._stuff[item]
   end
   player._stuff = {}
   restore_triggers()

   if player._save and not is_flooded(player._save.polygon) then
      player:position(player._save.x,
                      player._save.y,
                      player._save.z,
                      player._save.polygon)
      player.yaw = player._save.yaw
   end
end
